package binsearch;

/**
	Example of binary search routine.
	Adapted from Sommerville, Software Engineering, 1995, ch 23
*/

public class BinSearch 
{
	private int[] _elements;

	public BinSearch() {
		_elements = new int [] {};
	}

	public BinSearch(int[] init)
	{
		_elements = new int[init.length];
		System.arraycopy(init, 0, _elements, 0, init.length);
	}

	/**
		find v in sorted array elements (an instance variable) by binary search
	*/
	public boolean find(int key) throws assertionViolation
	{
		assert(isSorted()); // pre-condition
		
		// Trivially can't find key in an empty list
		if (isEmpty()) {
			return false;
		}
		
		int bottom = 0;
		int top = _elements.length-1;
		int lastIndex = (bottom+top)/2;
		int mid;

		boolean found = key == _elements[lastIndex];
		
		while ((bottom <= top) && !found) {
			assert(bottom <= top); // loop invariant
			mid = (bottom + top) / 2;
			found = key == _elements[mid];
			if (found) {
				lastIndex = mid;
			} else {
				if (_elements[mid] < key) {
					bottom = mid + 1;
				} else {
					top = mid - 1;
				}
			}
			// loop variant decreases: top - bottom
		}
		assert((key == _elements[lastIndex]) || !found); // post-condition
		return found;
	}
	
	public boolean isEmpty()
	{
		return _elements.length == 0;
	}

	public boolean isSorted()
	{
		int i;
		for (i=1;i<_elements.length;i++) {
			if (_elements[i-1] > _elements[i]) {
				return false;
			}
		}
		return true;
	}

	private void assert( boolean assertion)
		throws assertionViolation
	{
		if (! assertion) {
			throw new assertionViolation("BinSearch assertion failed");
		}
	}
}
