package binsearch;

/**
	Example of binary search test cases.
	Adapted from Sommerville, Software Engineering, 1995, ch 23
*/

public class TestBinSearch {
	// Main entry point
	static public void main(String[] args) 
	{
		(new TestBinSearch()).doTests();
	}

	public void doTests() {
		System.out.println("Starting BinSearch Tests");
		try {
			// Can't find anything in an empty list
			BinSearch eg0 = new BinSearch();
			assert(!eg0.find(17));

			// Unsorted lists are erroneous
			boolean unsortedCaught;
			try {
				unsortedCaught = false;
				BinSearch eg1 = new BinSearch(new int[] { 33, 20, 18, 17});
				eg1.find(17);
			} catch (assertionViolation err) {
				unsortedCaught = true;
			}
			assert(unsortedCaught);

			// List of length 1
			BinSearch eg2 = new BinSearch(new int[] { 17 });
			assert(eg2.find(17));
			assert(!eg2.find(0));
			assert(!eg2.find(50));

			// Various cases for longer list
			BinSearch eg3 = new BinSearch(new int[] { 17, 18, 20, 33 });
			assert(!eg3.find(0));
			assert(eg3.find(17));
			assert(eg3.find(33));
			assert(eg3.find(20));
			assert(!eg3.find(50));
			
		} catch (assertionViolation err) {
			err.printStackTrace();
		}
		System.out.println("Finished BinSearch Tests");
	}

	private void assert( boolean assertion)
		throws assertionViolation
	{
		if (! assertion) {
			throw new assertionViolation("TestBinSearch assertion failed");
		}
	}		
}
